@@ -14,7 +14,7 @@ from .n0struct_utils_compare import get__flag_compare_return_place
 from .n0struct_utils_compare import xpath_match
 from .n0struct_utils_compare import generate_composite_keys
 
-from .n0struct_files import n0eval
+from .n0struct_utils import n0eval
 from .n0struct_utils_find import split_name_index
 
 from .n0struct_n0list_ import n0list_
@@ -122,12 +122,12 @@ class n0list(n0list_):
                     # n0debug("return_lists")
                     # n0debug("xpath_found_str")
 
-                    if isinstance(next_parent_node, n0dict):
+                    if isinstance(next_parent_node, dict):
                         cur_parent_node, cur_node_name_index, cur_value, cur_found_xpath_str, \
-                            cur_not_found_xpath_list = n0dict._find(next_parent_node, xpath_list[1:], next_parent_node, return_lists)
-                    elif isinstance(next_parent_node, n0list):
+                            cur_not_found_xpath_list = n0dict._find(next_parent_node, xpath_list[1:], next_parent_node, return_lists,  xpath_found_str + f"[{i}]")
+                    elif isinstance(next_parent_node, (list, tuple)):
                         cur_parent_node, cur_node_name_index, cur_value, cur_found_xpath_str, \
-                            cur_not_found_xpath_list = self._find(xpath_list[1:], next_parent_node, return_lists)
+                            cur_not_found_xpath_list = self._find(xpath_list[1:], next_parent_node, return_lists, xpath_found_str + f"[{i}]")
                     else:
                         raise Exception("Unexpected type (%s) of %s" % (type(next_parent_node), str(next_parent_node)))
 
@@ -148,7 +148,7 @@ class n0list(n0list_):
                 except:
                     raise IndexError("Unknown index '%s[%s]'" % (xpath_found_str, node_index_str))
 
-                if isinstance(parent_node, (list, tuple, n0list)):
+                if isinstance(parent_node, (list, tuple)):
                     len__parent_node = len(parent_node)
                 else:
                     len__parent_node = 1
@@ -169,9 +169,9 @@ class n0list(n0list_):
                     # Deeper: any type under n0dict
                     #*******************************
                     next_parent_node =  parent_node[node_index_int]
-                    if isinstance(next_parent_node, n0dict):
+                    if isinstance(next_parent_node, dict):
                         return n0dict._find(next_parent_node, xpath_list[1:], next_parent_node, return_lists, "%s[%d]" % (xpath_found_str, node_index_int))
-                    if isinstance(next_parent_node, n0list):
+                    if isinstance(next_parent_node, (list, tuple)):
                         return self._find(xpath_list[1:], next_parent_node, return_lists, "%s[%d]" % (xpath_found_str, node_index_int))
                     else:
                         raise Exception("Unexpected type (%s) of %s" % (type(next_parent_node), str(next_parent_node)))
@@ -554,7 +554,7 @@ class n0list(n0list_):
                             )
                         )
 # 0.18 = Splitted logic, to avoid convertation of dict, OrderedDict into n0dict
-                    elif isinstance(self[self_i], (n0dict,)):
+                    elif isinstance(self[self_i], dict):
                         result.update_extend(
                             n0dict(self[self_i]).compare(
                                 # n0dict(other[other_i]),  # 0.18
@@ -1093,7 +1093,7 @@ class n0dict(n0dict_):
             # ..................................................................
             # Try to parse as list
             # ..................................................................
-            if isinstance(parent_node, (list, tuple, n0list)):
+            if isinstance(parent_node, (list, tuple)):
                 # *******************************
                 # Indulge #1 for incorrect syntax -- [*] was skipped for list in xpath
                 # *******************************
@@ -1202,14 +1202,14 @@ class n0dict(n0dict_):
             if node_index == "new()":
                 parent_node, node_name_index, cur_value, xpath_found_str, \
                     not_found_xpath_list = self._find(xpath_found_str, self, return_lists)
-                if not isinstance(parent_node[node_name_index], (list, tuple, n0list)):
+                if not isinstance(parent_node[node_name_index], (list, tuple)):
                     parent_node[node_name_index] = n0list([parent_node[node_name_index]])
                 return parent_node[node_name_index], None, None, xpath_found_str, ["[new()]"] + xpath_list[1:]
             # ..................................................................
             # Try to check all [*] items in the loop
             # ..................................................................
             elif node_index == "*":
-                if not isinstance(parent_node, (list, tuple, n0list)):
+                if not isinstance(parent_node, (list, tuple)):
                     # Hidden list. Convert single item into list
                     parent_node = [parent_node]
                 # cur_values = n0list([])
@@ -1268,7 +1268,7 @@ class n0dict(n0dict_):
                         #--------------------------------
                         return parent_node, None, None, xpath_found_str, xpath_list
                 else:
-                    if isinstance(parent_node, (list, tuple, n0list)) and len(parent_node):
+                    if isinstance(parent_node, (list, tuple)) and len(parent_node):
                         # *******************************
                         # Not correct: indulge #2 in incorrect syntax -- [*] was skipped for list in xpath
                         # *******************************
@@ -1297,7 +1297,7 @@ class n0dict(n0dict_):
                 except:
                     raise IndexError("Unknown index '%s[%s]'" % (xpath_found_str, node_index))
 
-                if isinstance(parent_node, (list, tuple, n0list)):
+                if isinstance(parent_node, (list, tuple)):
                     len__parent_node = len(parent_node)
                 else:
                     len__parent_node = 1
@@ -1351,14 +1351,19 @@ class n0dict(n0dict_):
                 if not next_node_name in parent_node:
                     # New node
                     if next_node_index:
-                        if next_node_index != "new()":
-                            raise Exception("Nonsence! Impossible to add %s[%s] to the list (%s)%s"
-                                            % (cur_node_name, cur_node_index, type(parent_node), str(parent_node)))
-                        # item[0] == None, will be reused at the next step with last()
+                        # NEWFIX
+                        if next_node_index != "new()" and next_node_index != "0":
+                            # raise Exception("Nonsence! Impossible to add %s[%s] to the list (%s)%s"
+                            #                 % (cur_node_name, cur_node_index, type(parent_node), str(parent_node)))
+                            raise Exception(f"Nonsence! Impossible to add {next_node_name}[{next_node_index}] to the {cur_node_name}"
+                                            + (f"[{cur_node_index}]" if cur_node_index else "")
+                                            + f" {str(parent_node)}"
+                            )
                         parent_node.update({next_node_name: n0list([None])})
                         next_node = parent_node[next_node_name]
                         # next_node_name_index = "[%s]" % next_node_index
                         next_node_name_index = "[last()]"
+                        # item[0] == None, will be reused at the next step with last()
                     else:
                         parent_node.update({next_node_name: n0dict({})})
                         next_node = parent_node
@@ -1381,8 +1386,13 @@ class n0dict(n0dict_):
             ####################################################################
             # Parent is LIST or should be a list
             ####################################################################
+            # Original code
             if cur_node_index == "new()":
-                if not isinstance(parent_node[cur_node_name], (list, tuple, n0list)):
+            # FIXME
+            # or (isinstance(parent_node, (list, tuple)) and n0eval(cur_node_index) == len(parent_node)) \
+            # or (isinstance(parent_node[cur_node_name], (list, tuple)) and n0eval(cur_node_index) == len(parent_node)[cur_node_name]):
+                if cur_node_name and not isinstance(parent_node[cur_node_name], (list, tuple, n0list)):
+                # if not isinstance(parent_node[cur_node_name], (list, tuple, n0list)):
                     ####################################################################
                     # Convert not LIST into the list
                     ####################################################################
@@ -1413,7 +1423,7 @@ class n0dict(n0dict_):
                     next_node_name_index = "[%s]" % str(next_node_index)
                 else:
                     raise Exception("Nonsence! Both next_node_name and next_node_index could NOT be empty")
-            if cur_node_index == "last()":
+            elif cur_node_index == "last()":
                 # Came from previous level: we create [None] and point to [last()] for exchange
                 if not isinstance(parent_node, (list, tuple, n0list)):
                     raise Exception("Nonsence! if index '%s' is set then (%s)%s must be n0list"
@@ -1437,9 +1447,15 @@ class n0dict(n0dict_):
                     next_node = parent_node[-1]
                     # Expected to have 'new()' in next_node_index, else it will be failed at the next step
                     next_node_name_index = "[%s]" % str(next_node_index)
+            # New fix
+            elif n0eval(cur_node_index) == len(parent_node):
+                parent_node.append(None)
+                next_node = parent_node
+                next_node_name_index = "[last()]"
             else:
-                raise Exception("Nonsence! Impossible to add %s[%s] to the list (%s)%s"
-                                % (cur_node_name, cur_node_index, type(parent_node), str(parent_node)))
+                # raise Exception("Nonsence! Impossible to add %s[%s] to the list (%s)%s"
+                #                 % (cur_node_name, cur_node_index, type(parent_node), str(parent_node)))
+                raise Exception(f"Nonsence! Impossible to add item {cur_node_index} to the list {str(parent_node)}")
 
         if len(xpath_list) == 1:
             return next_node, next_node_name_index
